import{j as e,S as t}from"./index-CULuikBf.js";function o(){return e.jsxs("div",{className:"flex flex-col items-center justify-center h-full text-center p-12 space-y-2",children:[e.jsx(t,{className:"text-primary w-24 h-24"}),e.jsx("h2",{className:"text-3xl font-bold text-text",children:"Welcome to Local Explorer"}),e.jsx("p",{className:"text-text-secondary text-sm font-light",children:"Select a resource from the sidebar to view & manage it."})]})}export{o as component};
