import{r as s,v as c,q as d}from"./index-CULuikBf.js";const p=s.forwardRef(function(e,n){const{render:B,className:b,disabled:t=!1,focusableWhenDisabled:o=!1,nativeButton:r=!0,...a}=e,{getButtonProps:u,buttonRef:f}=c({disabled:t,focusableWhenDisabled:o,native:r}),l=s.useMemo(()=>({disabled:t}),[t]);return d("button",e,{state:l,ref:[n,f],props:[a,u]})});export{p as B};
